/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2012 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.obs.api;

import static com.openexchange.obs.api.Tools.transform;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * {@link ResultParser}
 *
 * @author <a href="mailto:marcus.klein@open-xchange.com">Marcus Klein</a>
 */
public final class ResultParser {

    private static final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

    private ResultParser() {
        super();
    }

    public static Result[] parse(final InputStream is) throws BuildServiceException {
        final Document document;
        try {
            final DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(is);
            final NodeList childs = document.getChildNodes();
            for (int i = 0; i < childs.getLength(); i++) {
                final Node node = childs.item(i);
                if ("resultlist".equals(node.getNodeName())) {
                     return parseResultList(node);
                } else if (Node.TEXT_NODE == node.getNodeType()) {
                    continue;
                } else {
                    throw new BuildServiceException("Unknown node in build result \"" + node.getNodeName() + "\".");
                }
            }
        } catch (ParserConfigurationException e) {
            throw new BuildServiceException(e.getMessage(), e);
        } catch (SAXException e) {
            throw new BuildServiceException(e.getMessage(), e);
        } catch (IOException e) {
            throw new BuildServiceException(e.getMessage(), e);
        }
        throw new BuildServiceException("Unable to parse build result. " + transform(document));
    }

    private static Result[] parseResultList(Node resultList) throws BuildServiceException {
        final NodeList childs = resultList.getChildNodes();
        final List<Result> results = new ArrayList<Result>();
        for (int i = 0; i < childs.getLength(); i++) {
            final Node node = childs.item(i);
            if ("result".equals(node.getNodeName())) {
                results.add(parseResult(node));
            } else if (Node.TEXT_NODE == node.getNodeType()) {
                continue;
            } else {
                throw new BuildServiceException("Unknown node in result list \"" + node.getNodeName() + "\" \"" + node.getNodeValue() + "\".");
            }
        }
        return results.toArray(new Result[results.size()]);
    }

    private static Result parseResult(final Node resultNode) throws BuildServiceException {
        final NamedNodeMap attributes = resultNode.getAttributes();
        final String name = attributes.getNamedItem("repository").getNodeValue();
        String arch = attributes.getNamedItem("arch").getNodeValue();
        final String state = attributes.getNamedItem("state").getNodeValue();
        final Result result = new Result(new Repository(name, Architecture.parseArchitecture(arch)), State.parseState(state));
        final NodeList childs = resultNode.getChildNodes();
        for (int i = 0; i < childs.getLength(); i++) {
            final Node node = childs.item(i);
            if ("status".equals(node.getNodeName())) {
                result.add(parseStatus(node));
            } else if (Node.TEXT_NODE == node.getNodeType()) {
                continue;
            } else {
                throw new BuildServiceException("Unknown node in result \"" + node.getNodeName() + "\" \"" + node.getNodeValue() + "\".");
            }
        }
        return result;
    }

    private static PackageStatus parseStatus(final Node status) throws BuildServiceException {
        final NamedNodeMap attributes = status.getAttributes();
        final String name = attributes.getNamedItem("package").getNodeValue();
        final String code = attributes.getNamedItem("code").getNodeValue();
        final PackageStatus retval = new PackageStatus(name, Code.parseCode(code));
        final NodeList childs = status.getChildNodes();
        for (int i = 0; i < childs.getLength(); i++) {
            final Node node = childs.item(i);
            if ("details".equals(node.getNodeName())) {
                retval.setDetails(parseDetails(node));
            } else if (Node.TEXT_NODE == node.getNodeType()) {
                continue;
            } else {
                throw new BuildServiceException("Unknown node in result \"" + node.getNodeName() + "\" \"" + node.getNodeValue() + "\".");
            }
        }
        return retval;
    }

    private static String parseDetails(final Node details) {
        return details.getTextContent();
    }
}
